package com.cf.imes.module.system.dal.mysql.permission;

import com.cf.imes.framework.mybatis.core.mapper.BaseMapperX;
import com.cf.imes.framework.mybatis.core.query.LambdaQueryWrapperX;
import com.cf.imes.module.system.controller.admin.permission.vo.menu.MenuListReqVO;
import com.cf.imes.module.system.dal.dataobject.permission.MenuDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

@Mapper
public interface MenuMapper extends BaseMapperX<MenuDO> {

    default MenuDO selectByParentIdAndNameAndSourceType(Long parentId, String name, Integer sourceType) {
        return selectOne(MenuDO::getParentId, parentId, MenuDO::getName, name, MenuDO::getSourceType, sourceType);
    }

    default Long selectCountByParentId(Long parentId) {
        return selectCount(MenuDO::getParentId, parentId);
    }

    default List<MenuDO> selectList(MenuListReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<MenuDO>()
                .likeIfPresent(MenuDO::getName, reqVO.getName())
                .eqIfPresent(MenuDO::getStatus, reqVO.getStatus())
                .eqIfPresent(MenuDO::getVisible, reqVO.getVisiable())
                .eqIfPresent(MenuDO::getSourceType, reqVO.getSourceType()));
    }

    default List<MenuDO> selectListByPermission(String permission) {
        return selectList(MenuDO::getPermission, permission);
    }

    /**
     * 查询对应范围的菜单
     *
     * @param ids      菜单编号集合
     * @param isManage 是否是管理后台
     * @return
     */
    List<MenuDO> selectListIn(@Param("ids") Collection<Long> ids, @Param("isManage") boolean isManage, @Param("language") String language);
}
